/*
 * PM - 2017
 * 
 * Harpa laser
 *
 * STUDENT: Biciila Otilia-Teodora
 * GRUPA: 334CB
 */
 
#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>

#define NUM_NOTE		8
#define NUM_OCTAVE		3

#define OCTAVA1			0
#define OCTAVA2			1
#define OCTAVA3			2

#define DURATA_NOTA1	40
#define DURATA_NOTA2	80
#define DURATA_NOTA3	120

#define PRAG			4.5

/* Frecventele notelor muzicale pentru 3 octave diferite */
int frecventa_nota[NUM_OCTAVE][NUM_NOTE] = {
	{262, 294, 330, 349, 392, 440, 494, 523},
	{523, 587, 659, 698, 784, 880, 987, 1046},
	{1046, 1174, 1318, 1396, 1567, 1760, 1975, 2093}
};

/* Variabila globala pentru octava curenta */
int octava = 0;

/* Genereaza o nota folosind buzzer-ul. */
void buzzer_reda(int nota) {
	int i, j;
	int durata_nota = 0;
	/* Se calculeaza perioada notei muzicale in functie de frecventa */
	int perioada_nota = 1000000 / frecventa_nota[octava][nota];
	
	/* Se determina durata in functie de octava */
	if (octava == OCTAVA1)
		durata_nota = DURATA_NOTA1;
	else if (octava == OCTAVA2)
		durata_nota = DURATA_NOTA2;
	else if (octava == OCTAVA3)
		durata_nota = DURATA_NOTA3;

	/* Se reda nota muzicala */
	for (i = 0; i < durata_nota; i++) {
		PORTD &= ~(1 << PD5); 
		for (j = 0; j < perioada_nota / 2; ++j)
			_delay_us(1);

		PORTD |= (1 << PD5);
		for (j = 0; j < perioada_nota / 2; ++j)
			_delay_us(1);
	}		
	
}

/* Initializare ADC */
void ADC_init(void)
{
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

/* Obtinerea valorii ADC */
int ADC_get(uint8_t channel)
{
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));

	return ADC;
}

/* Conversia datelor ADC in volti */
double ADC_voltage(int raw)
{
	return raw * 5 / 1024.;
}

/* Initializarea porturilor microcontroller-ului */
void init_porturi()
{
	/* Se seteaza pinul PD5 ca iesire */
	DDRD |= (1 << PD5);
	
	PORTD |= (1 << PD5);
	
	/* Se seteaza pinii PB3, PB4 si PB5 ca intrari */
	DDRB &= ~(1 << PB3);
	DDRB &= ~(1 << PB4);
	DDRB &= ~(1 << PB5);
	
	/* Se activeaza rezistenta de pull-up pentru pinii PB3, PB4 si PB5 */
	PORTB |= (1 << PB3);
	PORTB |= (1 << PB4);
	PORTB |= (1 << PB5);

	/* Seteaza pinii PB0, PB1 si PB2 ca iesiri */
	DDRB |= (1 << PB0);
	DDRB |= (1 << PB1);
	DDRB |= (1 << PB2);

	/* Se aprinde LED-ul albastru - corespunator primei octave */
	PORTB |= (1 << PB0);
	
	/* Se sting celelalte doua LED-uri */
	PORTB &= ~(1 << PB1);
	PORTB &= ~(1 << PB2);
	
	/* Se seteaza pinii PA0-PA7 ca intrari */
	DDRA &= ~(1 << PA0);
	DDRA &= ~(1 << PA1);
	DDRA &= ~(1 << PA2);
	DDRA &= ~(1 << PA3);
	DDRA &= ~(1 << PA4);
	DDRA &= ~(1 << PA5);
	DDRA &= ~(1 << PA6);
	DDRA &= ~(1 << PA7);
	
	/* Se activeaza rezistenta de pull-up pentru pinii PA0-PA7 */
	PORTA |= (1 << PA0);
	PORTA |= (1 << PA1);
	PORTA |= (1 << PA2);
	PORTA |= (1 << PA3);
	PORTA |= (1 << PA4);
	PORTA |= (1 << PA5);
	PORTA |= (1 << PA6);
	PORTA |= (1 << PA7);
}

/* Se verifica daca vreuna dintre corzi a fost ciupita */
void verifica_note(void)
{
	int i = 0;
	double valoare;

	/* Pentru fiecare dintre corzi */
	for (i = 0; i < NUM_NOTE; ++i) {
		/* Se citeste valoarea tensiunii de pe canal */
		valoare = ADC_voltage(ADC_get(i));
		
		/* Daca aceasta se afla sub pragul stabilit */
		if (valoare < PRAG)
			/* Se reda nota corespunzatoare */
			buzzer_reda(i);
	}

}

void canta(void) 
{
	while (1) {
		/* Daca se apasa butonul conectat la PB5 */
		if (PINB & (1 << PB5)) {
			/* Aprind LED-ul albastru */
			PORTB |= (1 << PB0);
			
			/* Sting celelalte doua LED-uri */
			PORTB &= ~(1 << PB1);
			PORTB &= ~(1 << PB2);
			
			/* Schimb octava */
			octava = 0;
		}
		
		/* Daca se apasa butonul conectat la PB4 */
		if (PINB & (1 << PB4)) {
			/* Aprind LED-ul verde */
			PORTB &= ~(1 << PB0);
			
			/* Sting celelalte doua LED-uri */
			PORTB |= (1 << PB1);
			PORTB &= ~(1 << PB2);
			
			/* Schimb octava */
			octava = 1;
		}
		
		/* Daca se apasa butonul conectat la PB3 */
		if (PINB & (1 << PB3)) {
			/* Aprind LED-ul rosu */
			PORTB &= ~(1 << PB0);
			
			/* Sting celelalte doua LED-uri */
			PORTB &= ~(1 << PB1);
			PORTB |= (1 << PB2);
			
			/* Schimb octava */
			octava = 2;
		}
		
		/* Se verifica daca vreuna dintre corzi a fost ciupite */
		verifica_note();
	}
}

int main(void) {
	/* Initializarea porturilor */
	init_porturi();
	/* Initializare ADC*/
	ADC_init();
	
	/* Sa inceapa distractia :D */
	canta();
	
	return 0;
}
